
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
    HANDLE hProcess, hThread;
    DWORD PriorityClass, Priority;
    unsigned int time_low, time_high;
    unsigned __int64 freq, counter, limit;

    /* get handles */
    hProcess = GetCurrentProcess();
    hThread  = GetCurrentThread();

    /* get current priorities */
    PriorityClass = GetPriorityClass(hProcess);
    Priority      = GetThreadPriority(hThread);

    /* get frequency of performance counter */
    QueryPerformanceFrequency((LARGE_INTEGER *) &freq);

    /* set highest priority */
    if (!SetPriorityClass(hProcess, REALTIME_PRIORITY_CLASS))
    {
        MessageBox(0, "Unable to set REALTIME_PRIORITY_CLASS.", "Error", MB_ICONERROR | MB_OK);
        return 1;
    }
    if (!SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL))
    {
        SetPriorityClass(hProcess, PriorityClass);
        MessageBox(0, "Unable to set THREAD_PRIORITY_TIME_CRITICAL.", "Error", MB_ICONERROR | MB_OK);
        return 1;
    }

    /* give up time slice */
    Sleep(0);

    /* compute limit one second away */
    QueryPerformanceCounter((LARGE_INTEGER *) &limit);
    limit += freq;

    /* read time */
    __asm {
        rdtsc
        mov    time_low, eax
        mov    time_high, edx
    }

    /* loop until 1 second has elapsed */
    do {

        QueryPerformanceCounter((LARGE_INTEGER *) &counter);

    } while (counter < limit);

    /* compute time difference */
    __asm {
        rdtsc
        sub    eax, time_low
        sbb    edx, time_high
        mov    time_low, eax
        mov    time_high, edx
    }

    /* restore priority */
    SetThreadPriority(hThread, Priority);
    SetPriorityClass(hProcess, PriorityClass);

    /* show result */
    {
        char buffer[20];
        wsprintf(buffer, "%08X%08X\n", time_high, time_low);
        MessageBox(0, buffer, "Result", MB_OK);
    }

    return 0;
}
